<?php
/*
 Plugin Name: RentPress: Gravity Forms Add-on
 Plugin URI: https://RentPress.io/
 Version: 1.1.1
 Requires at least: 5.8
 Requires PHP: 7.2
 Author: 30 Lines
 Author URI: https://rentpress.io/
 Description: Seamlessly sends leads to multifamily CRMs with Gravity Forms. Support for: RentCafe, Entrata, Knock, Funnel, and Anyone Home.
 Text Domain: rentPress-wp-gravity-forms
 License: GPLv3
 GNU GPLv3 License Origin: http://www.gnu.org/licenses/gpl-3.0.html
*/

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if (!defined('GF_PLUGIN_DIR_PATH')) {
    require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
    add_action('admin_notices', 'rentpress_gf_addon_cant_activate_admin_notice');
    deactivate_plugins(str_replace(ABSPATH . 'wp-content/plugins/', "",dirname(__FILE__)) . '/rentPressGravityFormsWP.php');
}

// Globals
define('RENTPRESS_GF_ADDON_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('RENTPRESS_GF_ADDON_PLUGIN_VERSION', '1.1.1');

// Init
function rentpress_gf_addon_cant_activate_admin_notice()
{
    ?>
<div class="notice notice-error is-dismissible">
  <p><?php _e('<strong>Cannot activate RentPress: Gravity Forms Add-on</strong><br/><br/>
                This plugin requires RentPress 7.3 and Gravity Forms to active.
                Please install the required plugins and try again. <br/><br/>
                Learn more about <a target="_blank" href="https://via.30lines.com/xPdGhGjl">RentPress: Gravity Forms Add-on »</a>
                or <a href="/wp-admin/plugin-install.php?s=rentpress&tab=search&type=term">Download RentPress now »</a>'
            ); ?></p>
</div>
<?php
}

function init_rentPressGravityForms() {
    include_once(ABSPATH . 'wp-admin/includes/plugin.php');
    $hasRentpress = defined('RENTPRESS_PLUGIN_ADMIN_DIR') && (version_compare(RENTPRESS_PLUGIN_VERSION, '7.3.0', ">="));
    if ($hasRentpress && defined('GF_PLUGIN_DIR_PATH')) {
        if (!get_option('rentpress_flush_rewrite_rules_flag')) {
            add_option('rentpress_flush_rewrite_rules_flag', true);
        }
        if (!get_option('rentpress_gf_addon_plugin_version')) {
            add_option('rentpress_gf_addon_plugin_version', RENTPRESS_GF_ADDON_PLUGIN_VERSION);
        }
        if (! method_exists('GFForms', 'include_addon_framework')) {
            return;
        }
        include_once 'src/rentPressGravityForms/AddDefaultForm.php';
        include_once 'src/rentPressGravityForms/Leads.php';
        GFAddOn::register('rentPressGravityFormsLeads');
        if(is_admin()) {
            $addDefaultForm = new rentPressGravityForms_AddDefaultForm();
        }
    } else {
        add_action('admin_notices', 'rentpress_gf_addon_cant_activate_admin_notice');
        deactivate_plugins(str_replace(ABSPATH . 'wp-content/plugins/', "",dirname(__FILE__)) . '/rentPressGravityFormsWP.php');
    }
}

add_action('gform_loaded', 'init_rentPressGravityForms');

function rentpress_gravity_forms_log($message)
{
    if (is_array($message)) {
        $message = json_encode($message);
    }
    $file = fopen(RENTPRESS_GF_ADDON_PLUGIN_DIR . "/rentpress_gravity_forms_logs.log", "a");
    fwrite($file, "\n" . date('Y-m-d h:i:s') . " :: " . $message);
    fclose($file);
}