<?php
GFForms::include_feed_addon_framework();

class rentPressGravityFormsLeads extends GFFeedAddOn
{

    protected $_version = RENTPRESS_GF_ADDON_PLUGIN_VERSION;
    protected $_min_gravityforms_version = '1.9';
    protected $_slug = 'rentpress-gravity-forms';
    protected $_path = 'rentpress-gravity-forms-addon/rentPressGravityForms.php';
    protected $_full_path = __FILE__;
    protected $_title = 'RentPress: Gravity Forms Add-on';
    protected $_short_title = 'RentPress Forms';

    private $apiKey;
    private $apiUsername;
    private static $_instance = null;

    /**
     * Creating an instance of this object
     */
    public static function get_instance() {
        if (self::$_instance == null) {
            self::$_instance = new rentPressGravityFormsLeads();
        }

        return self::$_instance;
    }

    /**
     * Initializing the object
     */
    public function init() {
        parent::init();
        $this->check_legacy_field();
        $this->feed_settings_fields();
        $this->plugin_settings_fields();
        $this->standard_fields_for_feed_mapping();

        $rentpress_options = get_option('rentpress_options');
        $this->serviceUrl = 'https://toplineconnect.com/api/v1';
        $this->apiUsername = $rentpress_options['rentpress_api_credentials_section_username'];
        $this->apiKey = $rentpress_options['rentpress_api_credentials_section_api_token'];
        $this->urlOverride = '';
    }

    /**
     * Setting minimum plugin requirements for this plugin: Gravity Forms and RentPress are required
     */
    public function minimum_requirements()
    {
        return array(
            'plugins' => array(
                'gravityforms/gravityforms.php',
           )
       );
    }

    private function request($parameters = array())
    {
        $override = isset($this->urlOverride) ? $this->urlOverride : '';
        $service = isset($this->serviceUrl) ? $this->serviceUrl : '';
        $request = $override ? $override : $service;
        /* Send remote API request to RentPress */

        $results = wp_remote_post(
            $request . '/leads', // build request url
            array(
                'method' => 'POST',
                'sslverify' => false,
                'body' => $parameters,
                'compress' => true,
                'headers' => array(/* set token and username */
                    'X-Topline-Token' => $this->apiKey,
                    'X-Topline-User' => $this->apiUsername
               ),
                'timeout' => 60
           )
       );
        return isset($results['body']) ? $results['body'] : '';
    }

    public function scripts()
    {
        $scripts = array(
            array(
                'handle'    => 'feed_settings_js',
                'src'       => $this->get_base_url() . '/feedsettings.js',
                'version'   => $this->_version,
                'deps'      => array(),
                'in_footer' => false,
                'callback'  => array($this, 'localize_scripts'),
                'strings'   => array(),
                'enqueue'   => array(
                    array(
                        'admin_page' => array('form_settings', 'plugin_settings'),
                        'tab'        => 'rentpress-gravity-forms'
                   ),
               )
            ),
       );

        return array_merge(parent::scripts(), $scripts);
    }

    public function styles()
    {
        $styles = array(
            array(
                'handle'  => 'feed_settings_css',
                'src'     => $this->get_base_url() . '/feedsettings.css',
                'version' => $this->_version,
                'enqueue'   => array(
                    array(
                        'admin_page' => array('form_settings', 'plugin_settings'),
                        'tab'        => 'rentpress-gravity-forms'
                   ),
               )
            )
        );

        return array_merge(parent::styles(), $styles);
    }

    public function check_legacy_field()
    {
        $add_ons_settings = get_option("gravityformsaddon_rentpress-gravity-forms_settings");
        if (isset($add_ons_settings['importSource']) ? $add_ons_settings['importSource'] : '') {
            switch ($add_ons_settings['importSource']) {
                case 'rent-cafe':
                    if (!isset($add_ons_settings['RentCafeApiUsername']) && !isset($add_ons_settings['RentCafeApiPassword'])) {
                        $add_ons_settings['RentCafeApiUsername'] = $add_ons_settings['apiUsername'];
                        $add_ons_settings['RentCafeApiPassword'] = $add_ons_settings['apiPassword'];
                    }
                    break;

                case 'entrata':
                    if (!isset($add_ons_settings['EntrataApiUsername']) && !isset($add_ons_settings['EntrataApiPassword'])) {
                        $add_ons_settings['EntrataApiUsername'] = $add_ons_settings['apiUsername'];
                        $add_ons_settings['EntrataApiPassword'] = $add_ons_settings['apiPassword'];
                    }
                    break;

                case 'knock':
                    if (!isset($add_ons_settings['KnockApiUsername']) && !isset($add_ons_settings['KnockApiPassword'])) {
                        $add_ons_settings['KnockApiUsername'] = $add_ons_settings['apiUsername'];
                        $add_ons_settings['KnockApiPassword'] = $add_ons_settings['apiPassword'];
                    }
                    break;

                default:
                    // code...
                    break;
            }
            unset($add_ons_settings['importSource']);
            unset($add_ons_settings['apiUsername']);
            unset($add_ons_settings['apiPassword']);
            update_option("gravityformsaddon_rentpress-gravity-forms_settings", $add_ons_settings);
        }
    }

    /**
     * Once a form with a feed is submitted, sends all applicable fields to TLC
     */
    public function process_feed($feed, $entry, $form)
    {
        $contact_standard_fields = array_map('strval', $this->get_field_map_fields($feed, 'contactStandardFields'));
        $add_arg_logs = intval($this->get_plugin_setting('add_arg_logs'));
        $add_response_logs = intval($this->get_plugin_setting('add_response_logs'));

        // Check for a username and password
        switch ($feed['meta']['importSource']) {
            case 'rentcafe':
                $source_username = $this->get_plugin_setting('RentCafeApiUsername');
                $source_password = $this->get_plugin_setting('RentCafeApiPassword');
                $api_key = $this->get_plugin_setting('RentCafeApiKey');
                $company_code = $this->get_plugin_setting('RentCafeCompanyCode');
                $api_version = $this->get_plugin_setting('UseRentCafeV2API');
                $endpoint = "https://api.rentcafe.com/rentcafeapi.aspx?";

                if ($api_version) {
                    $endpoint = "https://basic.rentcafeapi.com/lead/createlead";
                }

                if ($api_version) {
                    $currentTime = time();
                    $tokenExpirationOffset = 28800;
                    if (!get_option("rentpress_leads_RC_v2_token")) {
                        add_option("rentpress_leads_RC_v2_token", "");
                    }
                    if (!get_option("rentpress_leads_RC_v2_token_time")) {
                        add_option("rentpress_leads_RC_v2_token_time", "");
                    }

                    $tokenEndpoint = "https://us-central1-rentpress-test-project.cloudfunctions.net/api/requestRCToken/new";
                    $rentpress_options['rentpress_api_credentials_section_api_token'] = $this->get_plugin_setting('DashboardApiKey');
                    $rentpress_options['rentpress_api_credentials_section_username'] = $this->get_plugin_setting('DashboardCompanyCode');
                    if (!get_option('rentpress_leads_RC_v2_token_time') ||
                        intval(get_option('rentpress_leads_RC_v2_token_time')) + $tokenExpirationOffset < $currentTime
                    ) {
                        $tokenResponse = wp_remote_request(
                            $tokenEndpoint,
                            array(
                                'method' => 'POST',
                                'sslverify' => false,
                                'body' => array(
                                  'companyID' => $rentpress_options['rentpress_api_credentials_section_username'],
                                  'websiteKey' => $rentpress_options['rentpress_api_credentials_section_api_token'],
                                ),
                                'compress' => true,
                                'headers' => array(
                                    'Content-Type' => 'application/x-www-form-urlencoded',
                                ),
                                'timeout' => 60,
                            )
                        );

                        $tokenResponseJson = json_decode($tokenResponse['body']);

                        if (isset($tokenResponseJson->access_token) ? $tokenResponseJson->access_token : "") {
                            update_option("rentpress_leads_RC_v2_token_time", $currentTime);
                            update_option("rentpress_leads_RC_v2_token", $tokenResponseJson->access_token);
                        }
                    }

                    $token = get_option('rentpress_leads_RC_v2_token') ;
                    $args = array(
                        "companyCode"       => $company_code,
                        "propertyCode"      => isset($contact_standard_fields['property_code']) ? $this->get_field_value($form, $entry, $contact_standard_fields['property_code']) : '',
                        'firstName'         => isset($contact_standard_fields['first_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['first_name']) : '',
                        'lastName'          => isset($contact_standard_fields['last_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['last_name']) : '',
                        'email'             => isset($contact_standard_fields['email_address']) ? $this->get_field_value($form, $entry, $contact_standard_fields['email_address']) : '',
                        'phone'             => isset($contact_standard_fields['phone']) ? $this->get_field_value($form, $entry, $contact_standard_fields['phone']) : '',
                        'message'           => isset($contact_standard_fields['message']) ? $this->get_field_value($form, $entry, $contact_standard_fields['message']) : '',
                        "source" => isset($contact_standard_fields['secondary_source']) ? $this->get_field_value($form, $entry, $contact_standard_fields['secondary_source']) : '',
                        'addr1'             => isset($contact_standard_fields['address_one']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_one']) : '',
                        'addr2'             => isset($contact_standard_fields['address_two']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_two']) : '',
                        'city'              => isset($contact_standard_fields['city']) ? $this->get_field_value($form, $entry, $contact_standard_fields['city']) : '',
                        'state'             => isset($contact_standard_fields['state']) ? $this->get_field_value($form, $entry, $contact_standard_fields['state']) : '',
                        'zipCode'           => isset($contact_standard_fields['zipcode']) ? $this->get_field_value($form, $entry, $contact_standard_fields['zipcode']) : '',
                    );
                } else {
                    $args = array(
                        'requestType'       => 'lead',
                        'source'            => 'rent-cafe',
                        'username'          => $source_username,
                        'password'          => $source_password,
                        'firstName'        => isset($contact_standard_fields['first_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['first_name']) : '',
                        'lastName'         => isset($contact_standard_fields['last_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['last_name']) : '',
                        'email'             => isset($contact_standard_fields['email_address']) ? $this->get_field_value($form, $entry, $contact_standard_fields['email_address']) : '',
                        'phone'             => isset($contact_standard_fields['phone']) ? $this->get_field_value($form, $entry, $contact_standard_fields['phone']) : '',
                        'message'           => isset($contact_standard_fields['message']) ? $this->get_field_value($form, $entry, $contact_standard_fields['message']) : '',
                        'propertyCode'     => isset($contact_standard_fields['property_code']) ? $this->get_field_value($form, $entry, $contact_standard_fields['property_code']) : '',
                        'secondarySource'  => isset($contact_standard_fields['secondary_source']) ? $this->get_field_value($form, $entry, $contact_standard_fields['secondary_source']) : '',
                        'addr1'       => isset($contact_standard_fields['address_one']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_one']) : '',
                        'addr2'       => isset($contact_standard_fields['address_two']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_two']) : '',
                        'city'              => isset($contact_standard_fields['city']) ? $this->get_field_value($form, $entry, $contact_standard_fields['city']) : '',
                        'state'             => isset($contact_standard_fields['state']) ? $this->get_field_value($form, $entry, $contact_standard_fields['state']) : '',
                        'ZIPCode'           => isset($contact_standard_fields['zipcode']) ? $this->get_field_value($form, $entry, $contact_standard_fields['zipcode']) : '',
                    );
                    $endpoint = $endpoint . http_build_query($args);
                }
                if ($api_key) {
                    $args['apiToken'] = $api_key;
                }
                if ($add_arg_logs) {
                    rentpress_gravity_forms_log('Args: ' . json_encode($args));
                }

                $result = wp_remote_post($endpoint,
                    array(
                        'method'      => $api_version ? 'POST' : 'GET',
                        'timeout'     => 45,
                        'redirection' => 5,
                        'httpversion' => '1.0',
                        'blocking'    => true,
                        'headers'     => $api_version ? array("Content-Type" => "application/json", "vendor" => $source_username, "accept" => "text/plain", "Authorization" => " Bearer " . $token) : array(),
                        'body'        => $api_version ? json_encode($args) : array(),
                        'cookies'     => array()
                   )
                );

                if ($add_response_logs) {
                    rentpress_gravity_forms_log('Endpoint With Args: ' . $endpoint);
                    rentpress_gravity_forms_log('Response: ' . json_encode($result));
                }

                if (isset($contact_standard_fields['api_response']) ? $contact_standard_fields['api_response'] : '' && $result) {
                    $data = array(
                        'field' => $contact_standard_fields['api_response'],
                        'response' => isset($result['body']) ? $result['body'] : "",
                   );
                    add_action('gform_after_submission', function ($entry) use ($data) {
                        GFAPI::update_entry_field($entry['id'] ,$data['field'], $data['response']);
                    }, 10, 2);
                }

                break;

            case 'entrata':
                $source_username = $this->get_plugin_setting('EntrataApiUsername');
                $source_password = $this->get_plugin_setting('EntrataApiPassword');
                $source_subdomaim = $this->get_plugin_setting('EntrataApiSubdomain');
                $encodedAuth =  base64_encode($source_username . ':' . $source_password);
                $useOldMethod = true;

                if (!(isset($source_username) ? $source_username : '') && !(isset($source_password) ? $source_password : '') && !(isset($source_subdomaim) ? $source_subdomaim : '')) {
                    break;
                }

                $endpoint = "https://". $source_subdomaim .".entrata.com/api/v1/leads";
                $firstName = $this->get_field_value($form, $entry, $contact_standard_fields['first_name']);
                $lastName = $this->get_field_value($form, $entry, $contact_standard_fields['last_name']);
                $email = $this->get_field_value($form, $entry, $contact_standard_fields['email_address']);
                $message = isset($contact_standard_fields['message']) ? $this->get_field_value($form, $entry, $contact_standard_fields['message']) : '';
                $phone = isset($contact_standard_fields['phone']) ? preg_replace("/[^A-Za-z0-9 ]/", '', $this->get_field_value($form, $entry, $contact_standard_fields['phone'])) : null;
                $moveDate = isset($contact_standard_fields['move_in_date']) ? $this->get_field_value($form, $entry, $contact_standard_fields['move_in_date']) : '';
                $propertyCode = isset($contact_standard_fields['property_code']) ? $this->get_field_value($form, $entry, $contact_standard_fields['property_code']) : '';
                $bedrooms = isset($contact_standard_fields['number_of_bedrooms']) ? $this->get_field_value($form, $entry, $contact_standard_fields['number_of_bedrooms']) : '';
                $maxBudget = isset($contact_standard_fields['maxBudget']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['maxBudget'])) : '';
                $floorplan = isset($contact_standard_fields['floor_plan_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['floor_plan_name']) : '';
                $unit = isset($contact_standard_fields['unit']) ? $this->get_field_value($form, $entry, $contact_standard_fields['unit']) : "";
                $occupants = isset($contact_standard_fields['occupants']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['occupants'])) : '1';
                $lead_source = isset($contact_standard_fields['lead_source']) ? $this->get_field_value($form, $entry, $contact_standard_fields['lead_source']) : 'website';
                $address_one = isset($contact_standard_fields['address_one']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_one']) : '';
                $address_two = isset($contact_standard_fields['address_two']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_two']) : '';
                $city =  isset($contact_standard_fields['city']) ? $this->get_field_value($form, $entry, $contact_standard_fields['city']) : '';
                $state = isset($contact_standard_fields['state']) ? $this->get_field_value($form, $entry, $contact_standard_fields['state']) : '';
                $zipcode = isset($contact_standard_fields['zipcode']) ? $this->get_field_value($form, $entry, $contact_standard_fields['zipcode']) : '';
                $nowDateTime = new DateTime("now", new DateTimeZone('US/Mountain'));
                $currentDateTime = $nowDateTime->format("Y-m-d") .  "T" .  $nowDateTime->format('h:i:s');

                if ($useOldMethod) {
                    $args = [
                        "auth" => [
                            "type" => "basic"
                        ],
                        "requestId" => "15",
                        "method" => [
                            "name" => "sendMitsLeads",
                            "params" => [
                                "propertyId" => $propertyCode,
                                "doNotSendConfirmationEmail" => "0",
                                "isWaitList" => "0",
                                "Prospects" => [
                                    "Prospect" => [
                                        [
                                            "LastUpdateDate" => $currentDateTime,
                                            "Customers" => [
                                                "Customer" => [
                                                    "Name" => [
                                                        "FirstName" => $firstName,
                                                        "LastName" => $lastName
                                                    ],
                                                    "Email" => $email
                                                ]
                                            ],
                                            "CustomerPreferences" => [
                                                "DesiredLeaseTerms" => "12",
                                                "NumberOfOccupants" => $occupants,
                                                "Comment" => $message
                                            ],
                                            "Events" => [
                                                "Event" => []
                                            ]
                                        ]
                                    ]
                                ]
                            ]
                        ]
                    ];

                    if ($address_one && $address_two && $city && $state && $zipcode) {
                        $args['method']['params']['Prospects']['Prospect'][0]['Customers']['Customer']['Address'] = [
                            "AddressLine1" => $address_one,
                            "AddressLine2" => $address_two,
                            "AddressLine3" => "",
                            "City" => $city,
                            "State" => $state,
                            "PostalCode" => $zipcode,
                            "@attributes" => [
                                "AddressType" => "current"
                            ]
                        ];
                    }

                    if ($phone) {
                        $args['method']['params']['Prospects']['Prospect'][0]['Customers']['Customer']['Phone'] = [
                            [
                                "PhoneNumber" => $phone,
                                "@attributes" => [
                                    "PhoneType" => "personal"
                                ]
                            ]
                        ];
                    }

                    if ($unit) {
                        $args['method']['params']['Prospects']['Prospect'][0]['CustomerPreferences']['DesiredUnit'] = [
                            "Identification" => [
                                "IDValue" => $unit,
                                "@attributes" => [
                                    "IDRank" => "primary",
                                    "IDScopeType" => "sender",
                                    "IDType" => "property_unit_id"
                                ]
                            ]
                        ];
                        if (!$floorplan) {
                            $args['method']['params']['Prospects']['Prospect'][0]['CustomerPreferences']['DesiredFloorplan'] = "No Floorplan Given";
                        }
                    }

                    if ($floorplan) {
                        $args['method']['params']['Prospects']['Prospect'][0]['CustomerPreferences']['DesiredFloorplan'] = $floorplan;
                    }

                    if ($moveDate) {
                        $args['method']['params']['Prospects']['Prospect'][0]['CustomerPreferences']['TargetMoveInDate'] = $moveDate;
                    }

                    if ($bedrooms) {
                        $args['method']['params']['Prospects']['Prospect'][0]['CustomerPreferences']['DesiredNumBedrooms'] = [
                            "@attributes" => [
                                "Exact" => $bedrooms
                            ]
                        ];
                    }

                    if ($maxBudget) {
                        $args['method']['params']['Prospects']['Prospect'][0]['CustomerPreferences']['DesiredRent'] = [
                            "@attributes" => [
                                "Max" => $maxBudget
                            ]
                        ];
                    }

                    if ($lead_source) {
                        $args['method']['params']['Prospects']['Prospect'][0]['TransactionData']['OriginatingLeadSource'] = $lead_source;
                    }

                } else {
                    $args = array(
                        "auth" => [
                            "type" => "basic"
                        ],
                        "requestId" => "15",
                        "method" => [
                            "name" => "sendLeads",
                            "params" => [
                                "propertyId" => $propertyCode,
                                "doNotSendConfirmationEmail" => "1",
                                "isWaitList" => "0",
                                "prospects" => [
                                    "prospect" => [
                                        "leadSource" => [
                                            "originatingLeadSourceId" => $lead_source,
                                            "additionalLeadSourceIds" => "xxxx,xxxx,xxxx"
                                         ],
                                        "createdDate" => "mm/dd/yyyyT03:47:33",
                                        "leasingAgentId" => "318487",
                                        "customers" => [
                                            "customer" => [
                                                "name" => [
                                                    "firstName" => $firstName,
                                                    "lastName" => $lastName,
                                                    "namePrefix" => "",
                                                    "middleName" => "",
                                                    "maidenName" => "",
                                                    "nameSuffix" => ""
                                                ],
                                                "customerRelationshipTypeId" => "XXXX",
                                                "address" => [
                                                    "addressLine1" => $address_one,
                                                    "addressLine2" => $address_two,
                                                    "addressLine3" => "",
                                                    "city" => $city,
                                                    "state" => $state,
                                                    "postalCode" => $zipcode
                                                ],
                                                "phone" => [
                                                    "personalPhoneNumber" => $phone,
                                                    "cellPhoneNumber" => "",
                                                    "officePhoneNumber" => "",
                                                    "faxNumber" => ""
                                                ],
                                                "email" => $email,
                                                "marketingPreferences" => [
                                                    "email" => [
                                                        "optInLeadCommunication" => "1",
                                                        "optInMessageCenter" => "0",
                                                        "optInContactPoints" => "1",
                                                        "optInMarketingMessages" => "0"
                                                    ],
                                                    "optInphone" => "1",
                                                    "optInMail" => "0"
                                                ]
                                            ]
                                        ],
                                        "customerPreferences" => [
                                            "desiredMoveInDate" => $moveDate,
                                            "desiredFloorplanId" => $floorplan,
                                            "desiredUnitTypeId" => "",
                                            "desiredUnitId" => $unit,
                                            "desiredRent" => [
                                                "min" => "",
                                                "max" => $maxBudget
                                            ],
                                            "desiredNumBedrooms" => $bedrooms,
                                            "desiredNumBathrooms" => "",
                                            "desiredLeaseTerms" => "",
                                            "desiredLeaseTermId" => "",
                                            "numberOfOccupants" => $occupants,
                                            "comment" => $message
                                        ],
                                        "events" => [
                                            "event" => []
                                        ]
                                    ]
                                ]
                            ]
                        ]
                    );
                }

                if ($add_arg_logs) {
                    rentpress_gravity_forms_log('Args: ' . json_encode($args));
                }

                if ($firstName && $lastName && $email && $propertyCode) {
                    $result = wp_remote_post($endpoint,
                        array(
                            'method'      => 'POST',
                            'timeout'     => 45,
                            'redirection' => 5,
                            'httpversion' => '1.0',
                            'blocking'    => true,
                            'headers'     => array(
                                'Authorization' => 'Basic ' . $encodedAuth,
                                'Content-Type' => 'application/json'
                            ),
                            'body'        => json_encode($args),
                            'cookies'     => array()
                        )
                    );

                    if ($add_response_logs) {
                        rentpress_gravity_forms_log('Response: ' . json_encode($result));
                    }
                    if (isset($contact_standard_fields['api_response']) ? $contact_standard_fields['api_response'] : '' && $result) {
                        $data = array(
                            'field' => $contact_standard_fields['api_response'],
                            'response' => json_encode($result),
                       );
                        add_action('gform_after_submission', function ($entry) use ($data) {
                            GFAPI::update_entry_field($entry['id'], $data['field'], $data['response']);
                        }, 10, 2);
                    }
                }
                break;

            case 'knock':
                $source_username = $this->get_plugin_setting('KnockApiUsername');
                $source_password = $this->get_plugin_setting('KnockApiPassword');

                if (!(isset($source_username) ? $source_username : '') && !(isset($source_password) ? $source_password : '')) {
                    break;
                }
                $API_KEY = $this->get_plugin_setting('KnockApiPassword');
                $endpoint = "https://syndication.knockrentals.com/prospect";
                $occupants = isset($contact_standard_fields['occupants']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['occupants'])) : '';
                $minBudget = isset($contact_standard_fields['minBudget']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['minBudget'])) : '';
                $maxBudget = isset($contact_standard_fields['maxBudget']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['maxBudget'])) : '';
                $communityId = isset($contact_standard_fields['community_id']) ? $this->get_field_value($form, $entry, $contact_standard_fields['community_id']) : '';
                $moveDate = isset($contact_standard_fields['move_in_date']) ? $this->get_field_value($form, $entry, $contact_standard_fields['move_in_date']) : '';
                $phoneNumber = isset($contact_standard_fields['phone']) ? preg_replace("/[^A-Za-z0-9 ]/", '', $this->get_field_value($form, $entry, $contact_standard_fields['phone'])) : null;
                $floorplan_bedrooms_text = [];
                $field = GFFormsModel::get_field($form, $contact_standard_fields['number_of_bedrooms']);

                if (isset($field->inputs)) {
                    foreach ($field->inputs as $input) {
                        $inputVal = $this->get_field_value($form, $entry, $input["id"]);
                        if (!empty($inputVal) || $inputVal == '0') {
                            switch ($inputVal) {
                                case '0':
                                    $floorplan_bedrooms_text[] = "STUDIO";
                                    break;

                                case '1':
                                    $floorplan_bedrooms_text[] = "1_BEDROOM";
                                    break;

                                case '2':
                                    $floorplan_bedrooms_text[] = "2_BEDROOMS";
                                    break;

                                case '3':
                                    $floorplan_bedrooms_text[] = "3_OR_MORE_BEDROOMS";
                                    break;
                                default:
                                    $floorplan_bedrooms_text[] = null;
                                    break;
                            }
                        }
                    }
                }

                $args = array(
                    "communityId"       => $communityId,
                    "sourceTitle"       => $this->get_plugin_setting('KnockApiUsername'),
                    "firstName"         => $this->get_field_value($form, $entry, $contact_standard_fields['first_name']),
                    "lastName"          => $this->get_field_value($form, $entry, $contact_standard_fields['last_name']),
                    "email"             => $this->get_field_value($form, $entry, $contact_standard_fields['email_address']),
                    "phone"             => $phoneNumber ? $phoneNumber : null,
                    "moveDate"          => isset($moveDate) && !is_null($moveDate) && $moveDate ? $moveDate : null,
                    "bedrooms"          => $floorplan_bedrooms_text,
                    "occupants"         => isset($occupants) && !is_null($occupants) && $occupants !== 0 ? intval($occupants) : null,
                    "leaseTermMonths"   => 12,
                    "minBudget"         => isset($minBudget) && !is_null($minBudget) && $minBudget !== 0 ? intval($minBudget) : null,
                    "maxBudget"         => isset($maxBudget) && !is_null($maxBudget) && $maxBudget !== 0 ? intval($maxBudget) : null,
                    "autorespond"       => true,
                    "message"           => $this->get_field_value($form, $entry, $contact_standard_fields['message']),
               );

                if ($add_arg_logs) {
                    rentpress_gravity_forms_log('Args: ' . json_encode($args));
                }

                $result = '';
                if ($communityId) {
                    $result = wp_remote_post($endpoint,
                        array(
                            'method'      => 'POST',
                            'timeout'     => 45,
                            'redirection' => 5,
                            'httpversion' => '1.0',
                            'blocking'    => true,
                            'headers'     => array(
                                "Content-Type" => "application/json",
                                "x-api-key" => $API_KEY
                           ),
                            'body'        => json_encode($args),
                            'cookies'     => array()
                       )
                    );
                    if ($add_response_logs) {
                        rentpress_gravity_forms_log('Response: ' . $result);
                    }
                    $responseToSave = $result['response']['code'] === 200 ? json_decode($result['body'])->id : $result['response']['code'];
                    if (isset($contact_standard_fields['api_response']) ? $contact_standard_fields['api_response'] : '' && $responseToSave) {
                        $data = array(
                            'field' => $contact_standard_fields['api_response'],
                            'response' => $responseToSave,
                       );
                        add_action('gform_after_submission', function ($entry) use ($data) {
                            GFAPI::update_entry_field($entry['id'], $data['field'], $data['response']);
                        }, 10, 2);
                    }
                }
                break;

            case 'funnel':
                $funnel_api_key = $this->get_plugin_setting('FunnelApiKey');

                if (!(isset($funnel_api_key) ? $funnel_api_key : '')) {
                    break;
                }
                $endpoint = "https://nestiolistings.com/api/v2/clients?key=" . $funnel_api_key;
                $discoveryEndpoint = "https://nestiolistings.com/api/v2/discovery-sources?key=" . $funnel_api_key;

                $firstName = isset($contact_standard_fields['first_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['first_name']) : '';
                $lastName = isset($contact_standard_fields['last_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['last_name']) : '';
                $email = isset($contact_standard_fields['email_address']) ? $this->get_field_value($form, $entry, $contact_standard_fields['email_address']) : '';
                $occupants = isset($contact_standard_fields['occupants']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['occupants'])) : '';
                $minBudget = isset($contact_standard_fields['minBudget']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['minBudget'])) : '';
                $maxBudget = isset($contact_standard_fields['maxBudget']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['maxBudget'])) : '';
                $communityId = isset($contact_standard_fields['community_id']) ? $this->get_field_value($form, $entry, $contact_standard_fields['community_id']) : '';
                $phoneNumber = isset($contact_standard_fields['phone']) ? preg_replace("/[^A-Za-z0-9 ]/", '', $this->get_field_value($form, $entry, $contact_standard_fields['phone'])) : null;
                $message = isset($contact_standard_fields['message']) ? $this->get_field_value($form, $entry, $contact_standard_fields['message']) : '';
                $clientReferral = isset($contact_standard_fields['client_referral']) ? $this->get_field_value($form, $entry, $contact_standard_fields['client_referral']) : "";
                $campaignId = isset($contact_standard_fields['campaign_id']) ? $this->get_field_value($form, $entry, $contact_standard_fields['campaign_id']) : "";
                $campaignInfo = isset($contact_standard_fields['campaign_info']) ? $this->get_field_value($form, $entry, $contact_standard_fields['campaign_info']) : "";
                $device =isset($contact_standard_fields['device']) ? $this->get_field_value($form, $entry, $contact_standard_fields['device']) : "";
                $sourceType = isset($contact_standard_fields['source_type']) ? $this->get_field_value($form, $entry, $contact_standard_fields['source_type']) : "";
                $leadSource = isset($contact_standard_fields['lead_source']) ? $this->get_field_value($form, $entry, $contact_standard_fields['lead_source']) : "";
                $unit = isset($contact_standard_fields['unit']) ? $this->get_field_value($form, $entry, $contact_standard_fields['unit']) : "";
                $communityId = isset($contact_standard_fields['community_id']) ? $this->get_field_value($form, $entry, $contact_standard_fields['community_id']) : "";
                $group = isset($contact_standard_fields['group']) ? $this->get_field_value($form, $entry, $contact_standard_fields['group']) : "";
                $floorplan_bedrooms_text = [];
                $field = GFFormsModel::get_field($form, $contact_standard_fields['number_of_bedrooms']);
                $moveDate = isset($contact_standard_fields['move_in_date']) ? $this->get_field_value($form, $entry, $contact_standard_fields['move_in_date']) : '';
                if ($moveDate) {
                    $moveDateTimeStamp = strtotime($moveDate);
                    $moveDate = date('Y-m-d', $moveDateTimeStamp);
                }
                $sms = isset($contact_standard_fields['sms_opt_in']) ? $this->get_field_value($form, $entry, $contact_standard_fields['sms_opt_in']) : "";
                if ($sms || isset($contact_standard_fields['sms_opt_in']) ? is_bool($contact_standard_fields['sms_opt_in']) : "") {
                    $sms_options = array(
                        "marketing-enabled",
                        "marketing-disabled",
                        "all-sms-disabled",
                        "client-opted-out",
                    );
                    if (is_bool($sms)) {
                        $sms = $sms ? "marketing-enabled" : "all-sms-disabled";
                    } elseif (in_array($sms, $sms_options)) {
                        break;
                    } else {
                        $sms = "";
                    }
                }

                if (isset($field->inputs)) {
                    foreach ($field->inputs as $input) {
                        $inputVal = $this->get_field_value($form, $entry, $input["id"]);
                        if (!empty($inputVal) || $inputVal == '0') {
                            switch ($inputVal) {
                                case '0':
                                    $floorplan_bedrooms_text[] = "studio";
                                    break;

                                case '1':
                                    $floorplan_bedrooms_text[] = "1br";
                                    break;

                                case '2':
                                    $floorplan_bedrooms_text[] = "2br";
                                    break;

                                case '3':
                                    $floorplan_bedrooms_text[] = "3br";
                                    break;

                                case '4':
                                    $floorplan_bedrooms_text[] = "4+br";
                                    break;

                                default:
                                    $floorplan_bedrooms_text[] = null;
                                    break;
                            }
                        }
                    }
                }

                $leadSourceMatch = null;
                if ($leadSource) {
                    $sourcelistCall = wp_remote_post(
                        $discoveryEndpoint,
                        array(
                            'method'      => 'GET',
                            'timeout'     => 45,
                            'redirection' => 5,
                            'httpversion' => '1.0',
                            'blocking'    => true,
                            'headers'     => array(),
                            'body'        => array(),
                            'cookies'     => array()
                       )
                   );
                    $sourcelist = isset($sourcelistCall["body"]) && $sourcelistCall["response"]["code"] == 200 ? json_decode($sourcelistCall["body"])->data->discovery_sources : "";
                    $leadSourceMatch = 0;
                    if (count($sourcelist)) {
                        foreach ($sourcelist as $source) {
                            if (strtolower($source->name) == strtolower($leadSource)) {
                                $leadSourceMatch++;
                            }
                        }
                    }
                    if (!$leadSourceMatch) {
                        $leadSource = null;
                    }
                }

                $args = [
                    "client" => [
                        "people" => [
                            [
                                "first_name" => $firstName,
                                "last_name" => $lastName,
                                "email" => $email,
                                "phone_1" => $phoneNumber,
                                "is_primary" => true
                            ]
                        ],
                        "move_in_date" => $moveDate,
                        "layout" => $floorplan_bedrooms_text,
                        "price_floor" => $minBudget,
                        "price_ceiling" => $maxBudget,
                        "notes" => $message,
                        "client_referral" => $clientReferral,
                        "campaign_id" => $campaignId,
                        "campaign_info" => $campaignInfo,
                        "device" => $device,
                        "source_type" => $sourceType,
                        "lead_source" => $leadSource,
                        "sms_opted_in" => $sms,
                        "community" => $communityId,
                        "unit" => $unit,
                        "group" => $group,
                    ]
                ];

                if ($add_arg_logs) {
                    rentpress_gravity_forms_log('Args: ' . json_encode($args));
                }

                $result = '';
                if ($firstName && $lastName && $email) {
                    $result = wp_remote_post($endpoint,
                        array(
                            'method'      => 'POST',
                            'timeout'     => 45,
                            'redirection' => 5,
                            'httpversion' => '1.0',
                            'blocking'    => true,
                            'headers'     => array(),
                            'body'        => json_encode($args),
                            'cookies'     => array()
                       )
                   );
                    if ($add_response_logs) {
                        rentpress_gravity_forms_log('Response: ' . $result['body']);
                    }
                    $responseToSave = $result['response']['code'] === 200 || $result['response']['code'] === 201 ? json_decode($result['body']) : $result['response']['code'];
                    if (isset($responseToSave->data->client->id)) {
                        $responseStr = "Client ID: " . $responseToSave->data->client->id;
                        if (isset($responseToSave->data->client->people[0]->id)) {
                            $responseStr .= " People ID: " . $responseToSave->data->client->people[0]->id;
                        }
                        $responseToSave = $responseStr;
                    }
                    if (isset($contact_standard_fields['api_response']) ? $contact_standard_fields['api_response'] : '' && $responseToSave) {
                        $data = array(
                            'field' => $contact_standard_fields['api_response'],
                            'response' => $responseToSave,
                       );
                        add_action('gform_after_submission', function ($entry) use ($data) {
                            GFAPI::update_entry_field($entry['id'], $data['field'], $data['response']);
                        }, 10, 2);
                    }
                    if (isset($contact_standard_fields['lead_source_flag']) ? $contact_standard_fields['lead_source_flag'] : '') {
                        $data = array(
                            'field' => $contact_standard_fields['lead_source_flag'],
                            'matchedSource' => $leadSourceMatch,
                       );
                        add_action('gform_after_submission', function($entry) use ($data) {
                            GFAPI::update_entry_field($entry['id'], $data['field'], $data['matchedSource']);
                        }, 10, 2);
                    }
                }
                break;


            case 'anyone_home':
                $source_username = $this->get_plugin_setting('AnyoneHomeApiUsername');
                $source_password = $this->get_plugin_setting('AnyoneHomeApiPassword');
                $source_key = $this->get_plugin_setting('AnyoneHomeApiKey');
                $auth = base64_encode($source_username . '/' . $source_key . ':' . $source_password);
                $endpoint_url = 'https://api.anyonehome.com/api/leadmanagement/importguestcard';


                // form start values //
                $firstName = $this->get_field_value($form, $entry, $contact_standard_fields['first_name']);
                $lastName = $this->get_field_value($form, $entry, $contact_standard_fields['last_name']);
                $email = $this->get_field_value($form, $entry, $contact_standard_fields['email_address']);
                $phone = isset($contact_standard_fields['phone']) ? preg_replace("/[^A-Za-z0-9 ]/", '', $this->get_field_value($form, $entry, $contact_standard_fields['phone'])) : null;
                $moveDate = isset($contact_standard_fields['move_in_date']) ? $this->get_field_value($form, $entry, $contact_standard_fields['move_in_date']) : '';
                $propertyCode = isset($contact_standard_fields['property_code']) ? $this->get_field_value($form, $entry, $contact_standard_fields['property_code']) : '';
                $bedrooms = isset($contact_standard_fields['number_of_bedrooms']) ? $this->get_field_value($form, $entry, $contact_standard_fields['number_of_bedrooms']) : '';
                $maxBudget = isset($contact_standard_fields['maxBudget']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['maxBudget'])) : '';
                $message = isset($contact_standard_fields['message']) ? $this->get_field_value($form, $entry, $contact_standard_fields['message']) : '';
                $floorplan = isset($contact_standard_fields['floor_plan_name']) ? $this->get_field_value($form, $entry, $contact_standard_fields['floor_plan_name']) : '';
                $occupants = isset($contact_standard_fields['occupants']) ? intval($this->get_field_value($form, $entry, $contact_standard_fields['occupants'])) : '';
                $propertyEmail = isset($contact_standard_fields['property_email']) ? $this->get_field_value($form, $entry, $contact_standard_fields['property_email']) : '';
                $lead_source = isset($contact_standard_fields['lead_source']) ? $this->get_field_value($form, $entry, $contact_standard_fields['lead_source']) : '';
                $address_one = isset($contact_standard_fields['address_one']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_one']) : '';
                $address_two = isset($contact_standard_fields['address_two']) ? $this->get_field_value($form, $entry, $contact_standard_fields['address_two']) : '';
                $address = $address_one . ' ' . $address_two;
                $city =  isset($contact_standard_fields['city']) ? $this->get_field_value($form, $entry, $contact_standard_fields['city']) : '';
                $state = isset($contact_standard_fields['state']) ? $this->get_field_value($form, $entry, $contact_standard_fields['state']) : '';
                $zipcode = isset($contact_standard_fields['zipcode']) ? $this->get_field_value($form, $entry, $contact_standard_fields['zipcode']) : '';

                $entryData = array(
                    "firstName" => $firstName,
                    "lastName" => $lastName,
                    "email" => $email,
                    "phone" => $phone,
                    "moveDate" => $moveDate,
                    "propertyCode" => $propertyCode,
                    "bedrooms" => $bedrooms,
                    "maxBudget" => $maxBudget,
                    "message" => $message,
                    "floorplan" => $floorplan,
                    "occupants" => $occupants,
                    "propertyEmail" => $propertyEmail,
                    "lead_source" => $lead_source,
                    "address" => $address,
               );

                $request_body = <<<XML
                    <LeadManagement>
                        <Prospect>
                            <TransactionData>
                                <OriginatingLeadSource>$lead_source</OriginatingLeadSource>
                                <ListingContactEmail>$propertyEmail</ListingContactEmail>
                            </TransactionData>
                            <Customers>
                                <Customer>
                                    <Property>
                                        <Vendor>30 Lines</Vendor>
                                        <PrimaryID>$propertyCode</PrimaryID>
                                    </Property>
                                    <Name>
                                        <FirstName>$firstName</FirstName>
                                        <LastName>$lastName</LastName>
                                    </Name>
                                    <Address>
                                        <Street>$address</Street>
                                        <City>$city</City>
                                        <State>$state</State>
                                        <ZipCode>$zipcode</ZipCode>
                                    </Address>
                                    <Contact>
                                        <Phone Type="cell">
                                            <Number></Number>
                                        </Phone>
                                        <Phone Type="home">
                                            <Number>$phone</Number>
                                        </Phone>
                                        <Email>$email</Email>
                                        <PreferredContactMethod>Phone</PreferredContactMethod>
                                        <ContactFrom></ContactFrom>
                                        <PrimaryID></PrimaryID>
                                        <Name>$firstName $lastName</Name>
                                        <ContactTo></ContactTo>
                                    </Contact>
                                </Customer>
                            </Customers>
                            <CustomerPreferences>
                                <TargetMoveInDate>$moveDate</TargetMoveInDate>
                                <Comments>$message</Comments>
                                <DesiredFloorplan>$floorplan</DesiredFloorplan>
                                <DesiredUnit>
                                    <Information>
                                        <UnitID></UnitID>
                                    </Information>
                                </DesiredUnit>
                                <DesiredNumBedrooms>$bedrooms</DesiredNumBedrooms>
                                <DesiredNumBathrooms></DesiredNumBathrooms>
                                <DesiredRent Max="$maxBudget" Min="0"/>
                                <DesiredLeaseTerms></DesiredLeaseTerms>
                                <TotalOccupants>$occupants</TotalOccupants>
                                <RequireResponse>true</RequireResponse>
                            </CustomerPreferences>
                            <Pets>
                                <Entry>
                                    <PetName></PetName>
                                    <PetType></PetType>
                                    <PetBreed></PetBreed>
                                </Entry>
                            </Pets>
                        </Prospect>
                    </LeadManagement>
XML;
                if ($add_arg_logs) {
                    rentpress_gravity_forms_log('Args: ' . json_encode($entryData));
                }

                if (!empty($lead_source) && !empty($propertyEmail) && !empty($propertyCode)) {
                    $response = wp_remote_post(
                        "$endpoint_url",
                        [
                            'headers' => [
                            'cache-control' => 'no-cache',
                            'Connection' => 'keep-alive',
                            'Accept-Encoding' => 'gzip, deflate',
                            'Host' => 'api.anyonehome.com',
                            'Cache-Control' => 'no-cache',
                            'Accept' => '*/*',
                            'Content-Type' => 'text/xml',
                            'Authorization' => 'Basic ' . $auth
                            ],
                        'body' => $request_body,
                        ]
                    );
                    $responseToSave = json_encode($response);

                    if ($add_response_logs) {
                        rentpress_gravity_forms_log('Response: ' . $responseToSave);
                    }

                    if (isset($contact_standard_fields['api_response']) ? $contact_standard_fields['api_response'] : '' && $responseToSave) {
                        $data = array(
                            'field' => $contact_standard_fields['api_response'],
                            'response' => $responseToSave,
                       );
                        add_action('gform_after_submission', function ($entry) use ($data) {
                            GFAPI::update_entry_field($entry['id'], $data['field'], $data['response']);
                        }, 10, 2);
                    }
                }

                break;

            default:
                $this->log->error('RentPress: Gravity Forms Leads: No import source selected.');
                break;
        }

    }

    /**
     * Creates the fields necessary for the plugin on the plugin settings page
     */
    public function plugin_settings_fields()
    {
        return array(
        //     array(
        //         'title'  => esc_html__('AppFolio Credentials', 'rentpress-wp-gravity-forms'),
        //         'description' => 'Enter the Client ID, Client Secret and Partner ID provided by AppFolio.',
        //         'fields' => array(
        //             array(
        //                 'label'             => esc_html__('AppFolio Client ID:', 'rentpress-wp-gravity-forms'),
        //                 'type'              => 'text',
        //                 'name'              => 'AppfolioClientId',
        //                 'class'             => 'medium',
        //                 'required'          => false,
        //             ),
        //             array(
        //                 'label'             => esc_html__('AppFolio Client Secret:', 'rentpress-wp-gravity-forms'),
        //                 'type'              => 'text',
        //                 'name'              => 'AppfolioClientSecret',
        //                 'class'             => 'medium',
        //                 'required'          => false,
        //             ),
        //             array(
        //                 'label'             => esc_html__('AppFolio Partner ID:', 'rentpress-wp-gravity-forms'),
        //                 'type'              => 'text',
        //                 'name'              => 'AppfolioPartnerId',
        //                 'class'             => 'medium',
        //                 'required'          => false,
        //            ),
        //        ),
        //    ),
            array(
                'title'  => esc_html__('RentCafe Credentials', 'rentpress-wp-gravity-forms'),
                'description' => 'Enter each of the following credentials to submit to RentCafe.',
                'fields' => array(
                    array(
                        'label'             => esc_html__('RentCafe API Token:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'RentCafeApiKey',
                        'tooltip'           => esc_html__('Five groups of letters and numbers separated by hyphens. Provided by RentCafe.', 'sometextdomain'),
                        'class'             => 'medium',
                        'required'          => false,
                    ),
                    array(
                        'label'             => esc_html__('RentCafe Company Code:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'RentCafeCompanyCode',
                        'tooltip'           => esc_html__('The letter "c" followed by a series of numbers. Provided by RentCafe.', 'sometextdomain'),
                        'class'             => 'medium',
                        // 'placeholder'       => 'c00001234',
                        'required'          => false,
                    ),
                    array(
                        'label'             => esc_html__('API Username:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'RentCafeApiUsername',
                        'tooltip'           => esc_html__('Provided by 30 Lines.', 'sometextdomain'),
                        'class'             => 'medium',
                        'required'          => false,
                    ),
                    array(
                        'label'             => esc_html__('RentCafe API Password:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'RentCafeApiPassword',
                        'class'             => 'medium',
                        'required'          => false,
                    ),
                    array(
                        'label'             => esc_html__('Dashboard Company ID:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'DashboardCompanyCode',
                        'tooltip'           => esc_html__('Provided by 30 Lines.', 'sometextdomain'),
                        'class'             => 'medium',
                        'required'          => false,
                    ),
                    array(
                        'label'             => esc_html__('Dashboard API Key:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'DashboardApiKey',
                        'tooltip'           => esc_html__('Provided by 30 Lines.', 'sometextdomain'),
                        'class'             => 'medium',
                        'required'          => false,
                    ),
                    array(
                        'label'             => esc_html__('RentCafe API Version', 'rentpress-wp-gravity-forms'),
                        'type'              => 'checkbox',
                        'name'              => 'RentCafeApiV2',
                        'class'             => 'medium',
                        'required'          => false,
                        'choices' => array(
                            array(
                                'label' => esc_html__('Use RentCafe V2 API', 'rentpress-wp-gravity-forms'),
                                'name'  => 'UseRentCafeV2API',
                            ),
                        ),
                   ),
               ),
           ),
            array(
                'title'  => esc_html__('Entrata Credentials', 'rentpress-wp-gravity-forms'),
                'description' => 'Enter the Username, Password, and Company Subdomain to submit to Entrata Lead Manager. <br /><br/>
                Typically the subdomain is just the name of the management company.
                To test to make sure you have the correct subdomain, try visiting this URL in your browser after swapping out the Xs:
                https://<strong>XXXXXXX</strong>.entrata.com/api/v1/documentation/ <br/>',
                'fields' => array(
                    array(
                        'label'             => esc_html__('Entrata API Username:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'EntrataApiUsername',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
                    array(
                        'label'             => esc_html__('Entrata API Password:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'EntrataApiPassword',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
                     array(
                        'label'             => esc_html__('Entrata API Subdomain:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'EntrataApiSubdomain',
                        'class'             => 'medium',
                        'tooltip'           => esc_html__( 'Only enter the company name part of the subdomain', 'sometextdomain' ),
                        'tooltip_class'     => 'tooltipclass',
                        'required'          => false,
                   ),
               ),
           ),
            array(
                'title'  => esc_html__('Knock Credentials', 'rentpress-wp-gravity-forms'),
                'description' => 'Enter the Username and Password provided by Knock.',
                'fields' => array(
                    array(
                        'label'             => esc_html__('Knock API Username:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'KnockApiUsername',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
                    array(
                        'label'             => esc_html__('Knock API Password:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'KnockApiPassword',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
               ),
           ),
            array(
                'title'  => esc_html__('Anyone Home Credentials', 'rentpress-wp-gravity-forms'),
                'description' => 'Enter the Username, Password, and API Key provided by Anyone Home.',
                'fields' => array(
                    array(
                        'label'             => esc_html__('Anyone Home API Username:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'AnyoneHomeApiUsername',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
                    array(
                        'label'             => esc_html__('Anyone Home API Password:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'AnyoneHomeApiPassword',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
                    array(
                        'label'             => esc_html__('Anyone Home API Key:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'AnyoneHomeApiKey',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
               ),
           ),
            array(
                'title'  => esc_html__('Funnel Credentials', 'rentpress-wp-gravity-forms'),
                'description' => 'Enter the Request URL and API Key provided by Funnel.',
                'fields' => array(
                    array(
                        'label'             => esc_html__('Funnel API Key:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'text',
                        'name'              => 'FunnelApiKey',
                        'class'             => 'medium',
                        'required'          => false,
                   ),
               ),
           ),
            array(
                'title'  => esc_html__('Logging Settings', 'rentpress-wp-gravity-forms'),
                'description' => 'These settings will enable logging of form submissions and responses from lead destinations.
                The logs will be created within your plugin files.
                They are only intended for troubleshooting. <br/>
                After troubleshooting, the files should be deleted and these settings turned off to return to normal operation.',
                'fields' => array(
                    array(
                        'label'             => esc_html__('Enable Logging', 'rentpress-wp-gravity-forms'),
                        'type'              => 'checkbox',
                        'name'              => 'EnableLogging',
                        'class'             => 'medium',
                        'required'          => false,
                        'choices' => array(
                            array(
                                'label' => 'Enable Submission Args Logging',
                                'name'  => 'add_arg_logs',
                                'value' => 1,
                           ),
                            array(
                                'label' => 'Enable Submission Response Logging',
                                'name'  => 'add_response_logs',
                                'value' => 1,
                           ),
                       ),
                   ),
               ),
           ),
       );
    }

    /**
     * Sets up fields for mapping form fields to 3rd party API fields in the form settings tab
     */
    public function standard_fields_for_feed_mapping()
    {
        $fields = [];

        return [
            array(
                'name'              => 'first_name',
                'label'             => esc_html__('First Name', 'rentpress-wp-gravity-forms'),
                'required'          => true,
                // 'field_type'        => array('name', 'text'),
                'default_value'     => $this->get_first_field_by_type('name', 3),
            ),
            array(
                'name'              => 'last_name',
                'label'             => esc_html__('Last Name', 'rentpress-wp-gravity-forms'),
                'required'          => true,
                // 'field_type'        => array('name', 'text'),
                'default_value'     => $this->get_first_field_by_type('name', 6),
            ),
            array(
                'name'              => 'email_address',
                'label'             => esc_html__('Email Address', 'rentpress-wp-gravity-forms'),
                'required'          => true,
                // 'field_type'        => array('email', 'text'),
                'default_value'     => $this->get_first_field_by_type('email'),
            ),
            array(
                'name'              => 'phone',
                'label'             => esc_html__('Phone:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('phone', 'text'),
                'required'          => false,
                'default_value'     => $this->get_first_field_by_type('phone'),
            ),
            array(
                'name'              => 'move_in_date',
                'label'             => esc_html__('Move-In Date:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('date', 'text'),
                'required'          => false
            ),
            array(
                'name'              => 'property_code',
                'label'             => esc_html__('Property Code: *', 'rentpress-wp-gravity-forms'),
                'field_type'        => array('text', 'hidden'),
                'class'             => 'medium',
                // 'required'          => true,
            ),
            array(
                'name'              => 'floor_plan_name',
                'label'             => esc_html__('Floor Plan Name:', 'rentpress-wp-gravity-forms'),
                'field_type'        => array('text', 'hidden'),
                'required'          => false
            ),
            array(
                'name'              => 'number_of_bedrooms',
                'label'             => esc_html__('Number of Bedrooms:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('checkbox', 'hidden'),
                'required'          => false
            ),
            array(
                'name'              => 'lead_source',
                'label'             => esc_html__('Lead Source:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('text', 'hidden'),
                'required'          => false,
            ),
            array(
                'name'              => 'message',
                'label'             => esc_html__('Message:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('textarea'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'primary_source',
                'label'             => esc_html__('Primary Source:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('text', 'hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'secondary_source',
                'label'             => esc_html__('Secondary Source:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('text', 'hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'address_one',
                'label'             => esc_html__('Address Line One:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('address','text'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'address_two',
                'label'             => esc_html__('Address Line Two:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('address','text'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'city',
                'label'             => esc_html__('City:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('address','text'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'state',
                'label'             => esc_html__('State:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('address','text'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'zipcode',
                'label'             => esc_html__('Zip Code:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('address','text'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'occupants',
                'label'             => esc_html__('Occupants:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('occupants','number'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'minBudget',
                'label'             => esc_html__('Min Budget:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('minBudget','number'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'maxBudget',
                'label'             => esc_html__('Max Budget:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('maxBudget','number'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'floorplan_code',
                'label'             => esc_html__('Floorplan Code:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('floorplan_code','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'floorplan_name',
                'label'             => esc_html__('Floorplan Name:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('floorplan_name','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'property_name',
                'label'             => esc_html__('Property Name:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('property_name','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'api_response',
                'label'             => esc_html__('API Response:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('api_response','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'lead_source_flag',
                'label'             => esc_html__('Lead Source Flag:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('lead_source_flag','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'community_id',
                'label'             => esc_html__('Community Id: *', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('community_id','hidden'),
                'class'             => 'medium',
                // 'required'          => true,
            ),
            array(
                'name'              => 'client_referral',
                'label'             => esc_html__('Client Referral:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('client_referral','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'campaign_id',
                'label'             => esc_html__('Campaign ID:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('campaign_id','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'campaign_info',
                'label'             => esc_html__('Campaign Info:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('campaign_info','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'device',
                'label'             => esc_html__('Device:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('device','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'source_type',
                'label'             => esc_html__('Source Type:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('source_type','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'unit',
                'label'             => esc_html__('Unit:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('unit','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'group',
                'label'             => esc_html__('Group:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('group','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'sms_opt_in',
                'label'             => esc_html__('SMS opt-in:', 'rentpress-wp-gravity-forms'),
                // 'field_type'        => array('group','hidden'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'property_email',
                'label'             => esc_html__('Property Email: *', 'rentpress-wp-gravity-forms'),
                'field_type'        => array('text','hidden','email'),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'has_cats',
                'label'             => esc_html__('Has Cats: ', 'rentpress-wp-gravity-forms'),
                'field_type'        => array(),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'has_dogs',
                'label'             => esc_html__('Has Dogs: ', 'rentpress-wp-gravity-forms'),
                'field_type'        => array(),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'has_other_pet',
                'label'             => esc_html__('Has Other Pet: ', 'rentpress-wp-gravity-forms'),
                'field_type'        => array(),
                'class'             => 'medium',
            ),
            array(
                'name'              => 'status',
                'label'             => esc_html__('Status: ', 'rentpress-wp-gravity-forms'),
                'field_type'        => array(),
                'class'             => 'medium',
            ),
        ];
    }

    /**
     * Sets up fields for form feed in form settings tab
     */
    public function feed_settings_fields()
    {
        $fields = [];
        if ($this->get_plugin_setting('importSource') == 'default') {
            $fields = array(
                'title' => esc_html__('Please select an import source on the RentPress Leads tab of the Gravity Forms settings page', 'rentpress-wp-gravity-forms'),
           );
        } else {
            $fields = array(
                'name'      => 'contactStandardFields',
                'label'     => esc_html__('Map Fields', 'rentpress-wp-gravity-forms'),
                'type'      => 'field_map',
                'field_map' => $this->standard_fields_for_feed_mapping(),
                'tooltip'   => '<h6>' . esc_html__('Map Fields', 'rentpress-wp-gravity-forms') . '</h6>' . esc_html__('Select which Gravity Form fields pair with their respective third-party service fields.', 'rentpress-wp-gravity-forms')
           );
        }
        return array(
            array(
                'title'  => esc_html__('Field Mapping Settings', 'rentpress-wp-gravity-forms'),
                'fields' => array(
                    array(
                        'label'   => esc_html__('Feed name', 'rentpress-wp-gravity-forms'),
                        'type'    => 'text',
                        'name'    => 'feedName',
                        'tooltip' => esc_html__('Enter a name to uniquely identify this setup.', 'rentpress-wp-gravity-forms'),
                        'class'   => 'medium',
                   ),
                    array(
                        'label'             => esc_html__('Select import source:', 'rentpress-wp-gravity-forms'),
                        'type'              => 'select',
                        'name'              => 'importSource',
                        'class'             => 'medium',
                        'choices'           => array(
                            array(
                                'label'     => esc_html__('Select an Import Source', 'rentpress-wp-gravity-forms'),
                                'value'     => 'default',
                           ),
                            array(
                                'label'     => esc_html__('RentCafe', 'rentpress-wp-gravity-forms'),
                                'value'     => 'rentcafe',
                           ),
                            array(
                                'label'     => esc_html__('Entrata', 'rentpress-wp-gravity-forms'),
                                'value'     => 'entrata',
                           ),
                            array(
                                'label'     => esc_html__('Knock', 'rentpress-wp-gravity-forms'),
                                'value'     => 'knock',
                           ),
                            array(
                                'label'     => esc_html__('Funnel', 'rentpress-wp-gravity-forms'),
                                'value'     => 'funnel',
                           ),
                            array(
                                'label'     => esc_html__('Anyone Home', 'rentpress-wp-gravity-forms'),
                                'value'     => 'anyone_home',
                           ),
                        //     array(
                        //         'label'     => esc_html__('AppFolio', 'rentpress-wp-gravity-forms'),
                        //         'value'     => 'appfolio',
                        //    ),
                       ),
                        'required'          => true,
                   ),
                   $fields,
               ),
           ),
       );
    }

    /**
     * Adds columns to the feed view, allowing us to see the name and edit/delete buttons for each feed
     */
    public function feed_list_columns()
    {
        return array(
            'feedName' => __('Name', 'rentpress-wp-gravity-forms')
       );
    }
}