document.addEventListener("DOMContentLoaded", function () {
  const importSourceInput = document.getElementById("importSource");
  const rentCafeVer = document.getElementById("userentcafev2api");

  if (importSourceInput) {
    const onLoadImportValue =
      importSourceInput.options[importSourceInput.selectedIndex].value;

    var standardContactFields = document.getElementById(
      "_gform_setting_contactStandardFields"
    ).value;
    if (standardContactFields) {
      standardContactFields = JSON.parse(standardContactFields);
      toggleMappingFields(onLoadImportValue, standardContactFields);
    }

    importSourceInput.addEventListener("change", (event) => {
      const importEl = event.target;
      const selectedImport = importEl.options[importEl.selectedIndex].value;
      if (selectedImport) {
        toggleMappingFields(selectedImport, standardContactFields);
      }
    });
  }

  if (rentCafeVer) {
    const passwordEl = document.getElementById("RentCafeApiPassword");
    const companyCodeEl = document.getElementById("RentCafeCompanyCode");
    const dashboardCodeEl = document.getElementById("DashboardCompanyCode");
    const dashboardKeyEl = document.getElementById("DashboardApiKey");

    if (rentCafeVer.checked) {
      passwordEl.parentNode.parentNode.style.display = "none";
      companyCodeEl.parentNode.parentNode.style.display = "inherit";
      dashboardCodeEl.parentNode.parentNode.style.display = "inherit";
      dashboardKeyEl.parentNode.parentNode.style.display = "inherit";
    } else {
      passwordEl.parentNode.parentNode.style.display = "inherit";
      companyCodeEl.parentNode.parentNode.style.display = "none";
      dashboardCodeEl.parentNode.parentNode.style.display = "none";
      dashboardKeyEl.parentNode.parentNode.style.display = "none";
    }

    rentCafeVer.addEventListener("change", (event) => {
      const verEl = event.target;
      if (verEl.checked) {
        passwordEl.parentNode.parentNode.style.display = "none";
        companyCodeEl.parentNode.parentNode.style.display = "inherit";
        dashboardCodeEl.parentNode.parentNode.style.display = "inherit";
        dashboardKeyEl.parentNode.parentNode.style.display = "inherit";
      } else {
        passwordEl.parentNode.parentNode.style.display = "inherit";
        companyCodeEl.parentNode.parentNode.style.display = "none";
        dashboardCodeEl.parentNode.parentNode.style.display = "none";
        dashboardKeyEl.parentNode.parentNode.style.display = "none";
      }
    });
  }
});

function toggleMappingFields(source, fields) {
  const fieldMapRows = document.getElementsByClassName(
    "gform-settings-generic-map__row"
  );

  for (var i = fieldMapRows.length - 1; i >= 0; i--) {
    row = fieldMapRows[i];
    row.style.display = "none";
  }

  switch (source) {
    case "knock":
      sourceFields = [
        "first_name",
        "last_name",
        "email_address",
        "phone",
        "message",
        "move_in_date",
        "number_of_bedrooms",
        "occupants",
        "minBudget",
        "maxBudget",
        "api_response",
        "community_id",
      ];
      break;

    case "entrata":
      sourceFields = [
        "first_name",
        "last_name",
        "email_address",
        "phone",
        "message",
        "move_in_date",
        "property_code",
        "number_of_bedrooms",
        "occupants",
        "floor_plan_name",
        "lead_source",
        "address_one",
        "address_two",
        "city",
        "state",
        "zipcode",
        "api_response",
        "unit",
      ];
      break;

    case "rentcafe":
      sourceFields = [
        "first_name",
        "last_name",
        "email_address",
        "phone",
        "message",
        "property_code",
        "primary_source",
        "secondary_source",
        "address_one",
        "address_two",
        "city",
        "state",
        "zipcode",
        "api_response",
      ];
      break;

    case "funnel":
      sourceFields = [
        "first_name",
        "last_name",
        "email_address",
        "phone",
        "message",
        "move_in_date",
        "number_of_bedrooms",
        "occupants",
        "minBudget",
        "maxBudget",
        "lead_source_flag",
        "community_id",
        "client_referral",
        "campaign_id",
        "campaign_info",
        "device",
        "source_type",
        "unit",
        "group",
        "lead_source",
        "api_response",
        "sms_opt_in",
      ];
      break;

    case "anyone_home":
      sourceFields = [
        "first_name",
        "last_name",
        "email_address",
        "phone",
        "message",
        "move_in_date",
        "property_code",
        "number_of_bedrooms",
        "occupants",
        "maxBudget",
        "address_one",
        "address_two",
        "city",
        "state",
        "zipcode",
        "lead_source",
        "property_email",
        "api_response",
      ];
      break;

    default:
      sourceFields = [];
  }

  for (var i = sourceFields.length - 1; i >= 0; i--) {
    fieldEl = document.getElementById(
      "_gform_setting_contactStandardFields_" + sourceFields[i]
    );
    if (fieldEl) {
      fieldEl.parentNode.parentNode.style.display = "inherit";
    }
  }
}
